#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:44:38 2024

@author: pteysseyre
"""

import numpy as np
import vlf4ions.sunrise_sunset as srst


def flags(amp, previous_nb_points, today, this_station, this_receiver, phase):
    """Flags 1 if the antenna is down, 2 if the transmitter is down, 3 if it is nighttime

    :param amp: Amplitude (pT)
    :param previous_nb_points: Number of points read last time
    :param today: Datetime, instant of file reading. Must be timezone-aware
    :param this_station: station class instance
    :param this_receiver: receiver class instance
    :param phase: Phase, before any filtering

    :return: - flag: flag indicating the state of the transmitter/receiver
        - length_data: number of points read this time
        - data_last_min: median total amplitude over the last minute"""

    flag = 0
    data_last_min = 0
    length_data = min(
        len(amp), len(phase)
    )  # this is necessary as there may be one more point in the phase than the amplitude, as it is read later

    # Check that we are in daytime
    daytime, sza = srst.is_daytime(
        this_station, this_receiver, today, this_station.sza_threshold
    )

    if daytime:
        # Antenna down: the number of data points in the last written file stays the same
        if length_data == previous_nb_points:
            flag = 1
        else:
            previous_nb_points = length_data

            if length_data > 60:
                amp = amp[-60:]
            else:
                amp = amp[-length_data:]

            data_last_min = np.median(amp)

        if (data_last_min < this_receiver.threshold and flag != 1) or np.abs(
            phase[-1] - phase[-2]
        ) > 20:

            # If the amplitude is too low, the station is not transmitting
            # If the phase is not well-decoded, the signal will be random and will vary greatly between two datapoints
            flag = 2
    else:
        flag = 3

    # We update the mean solar zenith angle in the path
    this_station.sza = sza

    return flag, length_data, data_last_min



import vlf4ions.forecast_nowcast as fn
import vlf4ions.class_definition as cd
import vlf4ions.compute_electron_density as ced
import numpy as np
import datetime as dt
import os

dirname = os.path.dirname(__file__)


def test_nowcast():
    """Test the nowcast function EXCEPT how alerts are sent"""

    # Create two test stations and the receiver
    st1 = cd.station("NRK", 0, 5, 24, 10, ":00", flag=0, quiet=1, sza=0, DP=0, DA=0)
    st2 = cd.station("GVT", 0, 5, 24, 10, ":00", flag=0, quiet=0)
    st3 = cd.station("T3", 0, 5, 24, 10, ":00", flag=1, quiet=0)
    rcv = cd.receiver("Receiver", 10, 10)

    # Create nowcast object
    ncst = fn.nowcast([st1, st2, st3], rcv, [])

    # Other parameters
    path_to_probas = os.path.join(dirname, "Data/")

    ncst.run(path_to_probas, path_to_probas, path_to_probas)

    error_Ne = np.abs(st1.Ne - ced.compute_Ne(70, 70.55, 0.395)) / ced.compute_Ne(
        70, 70.55, 0.395
    )  # Error on estimate of Ne (%)

    # The proba computation has already been tested elsewhere, here is it jsut to check that everything runs together

    assert ncst.nb_stations_on == 2 and error_Ne < 0.05


def test_plot_probas():
    """Test of the proba plot and in particular the threshold returned"""

    # Triangular 'proba' with peak at flux = 2 and total area equal to 1
    flux = np.linspace(1, 3, 10000)
    probas = np.zeros(np.shape(flux))
    probas[flux <= 2] = flux[flux <= 2] - 1
    probas[flux > 2] = -flux[flux > 2] + 3

    path_to_results = os.path.join(dirname, "Results/")
    today = dt.datetime.now()
    nb_stations_on = 2

    flux_notlog = np.log10(flux)

    ten, fifty, seventy, best = fn.plot_probas(
        probas, flux_notlog, path_to_results, today, nb_stations_on
    )

    # Check results
    errors = []

    if not np.abs(best - 2) < 0.01:
        errors.append("Plot_probas - Incorrect best guess")

    if not np.abs(best - fifty) < 0.01:
        errors.append("Plot_probas - Incorrect 50%")

    # Check 10%
    id = np.argmin(np.abs(ten - flux))
    area_10 = (3 - ten) * probas[id] / 2

    if not np.abs(area_10 - 0.1) < 0.01:
        errors.append("Plot_probas - Incorrect 10%")

    # Check 75%
    id = np.argmin(np.abs(seventy - flux))
    area_75 = (seventy - 1) * probas[id] / 2

    if not np.abs(area_75 - 0.25) < 0.01:
        errors.append("Plot_probas - Incorrect 75%")

    assert not errors, "errors occured:\n{}".format("\n".join(errors))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr  1 17:10:28 2025

@author: pteysseyre
"""

import pandas as pd
import datetime as dt
from datetime import timedelta, datetime
from pysolar.solar import get_altitude
import pytz
import numpy as np

""" This file deals with checking that we are in daytime.
It is based on previously computed sunrise and sunset times from https://nrc.canada.ca/en/research-development/products-services/software-applications/sun-calculator/

As a precaution, we define nighttime as between from one hour before sunset to one hour after sunrise.


History:
------------------

Updated in April 2025 to include calculation of the solar zenith angle


"""


def get_sza(today, lat, lon):
    """Returns the solar zenith angle (in rad) at the latitude/longitude given in input

    :param today: Datetime for the time of interest
    :param lat: Latitude of interest (in °)
    :param lon: Longitude of interest (in °)

    :return: sza, solar zenith angle (in rad)"""

    sza = float(90) - get_altitude(lat, lon, today)
    sza = sza * np.pi / 180

    return sza


def get_sunrise_sunset(today, path):
    """Returns the sunrise (civil + 1 hr) and sunset (civil - 1 hr) times (in UT)

    :param today: datetime (may only contain the year, month and day)
    :param path: path to the precomputed sunrise/sunset times

    :return: sunrise and sunset times for this day (in UT)"""

    print("Depreciated after version 1.3.4.post6")

    # Must be an easier way to do this, but get the year, month, and dayofyear of today.
    doy = today - dt.datetime(today.year, 1, 1, 0, 0, 0, 0, pytz.UTC)
    doy = doy.days

    # Get the sunrise / sunset time for that particular day
    file_sunrise = pd.read_csv(path, header=1)

    sunrise = file_sunrise.iat[doy - 1, 3]
    sunrise = datetime.strptime(sunrise, "%H:%M") + timedelta(hours=1)
    sunrise = sunrise.time()

    sunset = file_sunrise.iat[doy - 1, 5]
    sunset = datetime.strptime(sunset, "%H:%M") - timedelta(hours=1)
    sunset = sunset.time()

    return sunrise, sunset


def check_if_daytime(today, path):
    """Check whether the current time is greater than the sunrise time and less than sunset
    for this particular station and day

    :param today: datetime (may only contain the year, month and day)
    :param path: path to the precomputed sunrise/sunset times

    :return: Boolean, if we are in daytime
    """

    print("Depreciated after version 1.3.4.post6")

    sunrise, sunset = get_sunrise_sunset(today, path)

    return today.time() > sunrise and today.time() < sunset


def is_daytime(this_station, this_receiver, today, sza_threshold):
    """Check if the solar zenith angle at the station and the receiver is less than sza_threshold.
    Return True if this is the case, signaling that it is daytime

    :param this_station: Station class instance
    :param this_receiver: Receiver class instance
    :param today: Datetime, time of the computation. Must be timezone-aware
    :param sza_threshold: Solar zenith angle threshold below which it is daytime (Default: 85°)

    :returns: - daytime: Boolean, is true if it is daytime.
        - Sza, mean solar zenith angle (in rad) between the transmitter and the receiver
    """

    sza_station = get_sza(today, this_station.lat, this_station.lon)
    this_station.sza = sza_station
    sza_receiver = get_sza(today, this_receiver.lat, this_receiver.lon)

    sza = (sza_station + sza_receiver) / 2

    daytime = True

    if (
        np.max(np.array([np.abs(sza_station), np.abs(sza_receiver)]))
        > sza_threshold * np.pi / 180
    ):

        daytime = False

    return daytime, sza



import vlf4ions.read_files as rf
import os
import numpy as np

dirname = os.path.dirname(__file__)


def test_read_Narrowband():

    # Parameters
    la_date = "2024_05_02"
    la_station = "Test"
    path = os.path.join(dirname, "Data/")

    amp_NS, amp_EW, phase_NS, phase_EW, the_time = rf.read_Narrowband_postprocessing(
        la_date, la_station, path
    )

    errors = []

    # Check the time
    if not (
        np.size(the_time) == 86400
        and the_time[0] == 0
        and np.abs(the_time[1] - the_time[0] - 1 / 3600) < 0.0001
    ):
        errors.append("Read_files - Problem with the_time")

    print(amp_NS[50000])

    # Check other arrays
    if not (
        np.abs(amp_NS[50000] - 0.8791) < 0.01
        and np.abs(phase_NS[50000] + 104.6759) < 0.01
        and np.abs(amp_EW[50000] - 2.5119) < 0.01
        and np.abs(phase_EW[50000] - 83.4476) < 0.01
    ):
        errors.append("Read_files - Problem with the data")

    assert not errors, "errors occured:\n{}".format("\n".join(errors))



import vlf4ions.analyse_change_points as acp
from vlf4ions.class_definition import receiver
import numpy as np


def test_analyse_bp():

    # Common parameters to all tests
    detection_threshold = 100
    amp = 2
    the_time = np.arange(0, 0.5, 0.1)
    quiet_phase = 1
    quiet_amp = 1
    last_time = 0.2
    quiet_p1 = 0
    quiet_breakpoint = 0.2
    quiet = 1
    this_receiver = receiver("Test", 0, 0, 0.25)

    # Keep track of errors
    errors = []

    # Test 1 - beginning of the flare
    current_time = the_time[-1]
    current_p1 = 120
    phase = np.array([1, 1, 1, 1, 2])
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        quiet_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if quiet_now == 1 or DP_now != 1:
        errors.append("Problem with breakpoint analysis for flare onset")

    # Prepare for test 2
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.4
    phase = np.array([1, 1, 1, 1, 2, 2])
    the_time = np.arange(0, 6, 1) / 10

    # Test 2 - Flare maximum
    current_p1 = 50
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if quiet_now == 1 or DP_now != 1:
        errors.append("Problem with breakpoint analysis for flare maximum")

    # Prepare for test 3
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.5
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9])
    the_time = np.arange(0, 8, 1) / 10

    # Test 3 - Flare decay
    current_p1 = -10
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if quiet_now == 1:
        errors.append(
            "Problem with breakpoint analysis for flare decay: declared as quiet"
        )
    if np.abs(prev_timedecay - 0.55) > 0.1:
        errors.append("Problem with breakpoint analysis for flare decay: decay time")

    # Prepare for test 4
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.7
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7])
    the_time = np.arange(0, 10, 1) / 10

    # Test 4 - Flare decay update
    current_p1 = -5
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0.7:
        errors.append("Problem with flare decay update: wrong DP")
    if quiet_now == 1:
        errors.append(
            "Problem with breakpoint analysis for flare decay update: declared as quiet"
        )
    if np.abs(prev_timedecay - 0.85) > 0.1:
        errors.append(
            "Problem with breakpoint analysis for flare decay update: decay time"
        )

    # Prepare for test 5
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3])
    the_time = np.arange(0, 12, 1) / 10

    # Test 5 - Back to quiet times
    current_p1 = 10
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - back to quiet: DP is not zero")
    if quiet_now == 0:
        errors.append("Problem with bp analysis - does not go back to quiet ")

    # Test 6 - Quiet after quiet
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1])
    the_time = np.arange(0, 13, 1) / 10
    current_p1 = 0
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - quiet after quiet, DP not zeros")
    if quiet_now == 0:
        errors.append("Problem with bp analysis - does not go stay quiet ")

    # Test 7 - Test last quiet was too long ago
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 0.9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1, 1])
    the_time = np.arange(0, 14, 1) / 10
    current_p1 = -10
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0:
        errors.append(
            "Problem with bp analysis - quiet after quiet, DP not zero if slope is negative"
        )
    if quiet_now == 0:
        errors.append(
            "Problem with bp analysis - does not go stay quiet if new slope is negative"
        )

    # Test 8 - Quiet after quiet, with negative slope
    last_p1 = current_p1
    quiet = 0  # It is first detected as a flare
    last_time = 9  # Last quiet time
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1, 1])
    the_time = np.arange(0, 14, 1)
    current_p1 = 1
    current_time = 14  # The time now (14 - 9 > 3)
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - quiet after two hours, DP not zero")
    if quiet_now == 0:
        errors.append(
            "Problem with bp analysis - quiet after quiet, quiet_now is not 1"
        )

    # Test 9 - Quiet before DA is too small
    last_p1 = current_p1
    quiet = 0  # It is first detected as a flare
    last_time = 9  # Last quiet time
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1, 1])
    the_time = np.arange(0, 14, 1)
    current_p1 = 1
    current_time = 10  # The time now (14 - 9 > 3)
    detection_threshold = 0.5
    this_receiver.threshold = 10
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
        this_receiver,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - DA low, DP not zero")

    # assert no error message has been registered, else print messages
    assert not errors, "errors occured:\n{}".format("\n".join(errors))
